/*******************************************************************************
 * Copyright 2013 Luca De Petrillo
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package com.example.testandroid;

import java.util.ArrayList;
import java.util.LinkedList;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract;

import com.lukasoft.android.voicecontrol.api.VoiceControlPluginApi;
import com.lukasoft.android.voicecontrol.api.data.ContactData;

public class PluginActionReceiver extends BroadcastReceiver {
	private static final String LOG_TAG = "PluginActionReceiver";
	
	private static final String CHOICE_ELEMENT = "SELECTED_ELEMENT";

	private static final String PARAM_TEXT_TO_REPEAT = "TEXT_TO_REPEAT";

	private static final String PARAM_CONTACT = "CONTACT";
	private static final String CHOICE_CONTACT = "SELECTED_CONTACT";
	
	private static final String PARAM_CUSTOM = "CUSTOM_PARAM";

	@Override
	public void onReceive(Context context, Intent intent) {
		// Always validate Voice Control permission.
		/*if (!VoiceControlPluginApi.isPermissionAuthentic(context, VoiceControlPluginApi.PERMISSION_EXECUTE_TASK)) {
			// Some application is hacking this app!!! This should really never happen...
			ApplicationInfo hackishAppInfo = 
					VoiceControlPluginApi.getAppWithPermission(context, VoiceControlPluginApi.PERMISSION_EXECUTE_TASK);
			CharSequence hackishAppName = hackishAppInfo.loadLabel(context.getPackageManager());
			Log.e(LOG_TAG, String.format("Hack attempt from '%s' using Voice Control plugin APIs. Ignoring request.", 
					hackishAppName));
		} else {
		*/
			String actionId = intent.getStringExtra(VoiceControlPluginApi.EXTRA_ACTION_ID);
			
			// Handle each action with a dedicated method.
			if ("parrotAction".equals(actionId)) {
				handleParrotAction(intent);
	
			} else if ("choiceAction".equals(actionId)) {
				handleChoiceAction(context, intent);
	
			} else if ("contactAction".equals(actionId)) {
				handleContactAction(context, intent);
	
			} else if ("customDataTypeAction".equals(actionId)) {
				handleCustomDataTypeAction(context, intent);
				
			} else {
				// Should never happen...
				throw new IllegalAccessError(String.format("Action id '%s' not handled by this plugin!", actionId));
			}
		//}
	}

	public void handleParrotAction(Intent intent) {
		Bundle resultExtras = getResultExtras(true);

		Bundle parameters = intent.getBundleExtra(VoiceControlPluginApi.EXTRA_PARAMETERS);

		// Lets check if we have something to repeat.
		if (!parameters.containsKey(PARAM_TEXT_TO_REPEAT)) {
			// The user didn't said the text to repeat. We tell to Voice Control to ask for it.
			setResultCode(VoiceControlPluginApi.ACTION_RESULT_ASK_PARAM);

			resultExtras.putString(VoiceControlPluginApi.EXTRA_PARAMETER_ID, PARAM_TEXT_TO_REPEAT);

			// Quit and wait for the next invocation that will have the needed parameter.
			return;
		}

		String textToRepeat = parameters.getString(PARAM_TEXT_TO_REPEAT);

		// Setting the result Bundle and result code for this BroadcastReceiver execution.
		setResultCode(VoiceControlPluginApi.ACTION_RESULT_OK);

		// We just tell to Voice Control to say back what the user said, setting it to the message in the result bundle.
		resultExtras.putString(VoiceControlPluginApi.EXTRA_MESSAGE, textToRepeat);
	}

	public void handleChoiceAction(Context context, Intent intent) {
		Bundle resultExtras = getResultExtras(true);

		Bundle choices = intent.getBundleExtra(VoiceControlPluginApi.EXTRA_CHOICES);

		// Lets check if the user selected something.
		if (!choices.containsKey(CHOICE_ELEMENT)) {
			// User didn't selected something yet. Build data to ask for it.
			setResultCode(VoiceControlPluginApi.ACTION_RESULT_ASK_CHOICE);

			resultExtras.putString(VoiceControlPluginApi.EXTRA_CHOICE_ID, CHOICE_ELEMENT);
			resultExtras.putString(VoiceControlPluginApi.EXTRA_MESSAGE,
					context.getString(R.string.action_choice_element_request));
			resultExtras.putStringArray(
					VoiceControlPluginApi.EXTRA_CHOICE_LABELS,
					new String[] { 
							context.getString(R.string.action_choice_element_label, 1),
							context.getString(R.string.action_choice_element_label, 2),
							context.getString(R.string.action_choice_element_label, 3),
							context.getString(R.string.action_choice_element_label, 4),
							context.getString(R.string.action_choice_element_label, 5) 
						});

			resultExtras.putLongArray(VoiceControlPluginApi.EXTRA_CHOICE_VALUES, new long[] { 1L, 2L, 3L, 4L, 5L });

			// Quit and wait for the next invocation.
			return;
		}

		long element = choices.getLong(CHOICE_ELEMENT);

		setResultCode(VoiceControlPluginApi.ACTION_RESULT_OK);

		resultExtras.putString(VoiceControlPluginApi.EXTRA_MESSAGE,
				context.getString(R.string.action_choice_message, element));
	}

	public void handleContactAction(Context context, Intent intent) {
		Bundle resultExtras = getResultExtras(true);

		Bundle parameters = intent.getBundleExtra(VoiceControlPluginApi.EXTRA_PARAMETERS);
		Bundle choices = intent.getBundleExtra(VoiceControlPluginApi.EXTRA_CHOICES);

		// If the contact parameter is missing, ask for it.
		if (!parameters.containsKey(PARAM_CONTACT)) {
			setResultCode(VoiceControlPluginApi.ACTION_RESULT_ASK_PARAM);

			resultExtras.putString(VoiceControlPluginApi.EXTRA_PARAMETER_ID, PARAM_CONTACT);

			return;
		}

		ArrayList<Bundle> contacts = parameters.getParcelableArrayList(PARAM_CONTACT);

		int contactId;
		
		// Use the contact on the parameter of it is only one.
		if (contacts.size() == 1) {
			contactId = contacts.get(0).getInt(ContactData.CONTACT_ID);

		// We have more than one result for the contact name. If we already asked the user to select a contact, we use the
		// selected one.
		} else if (choices.containsKey(CHOICE_CONTACT)) {
				contactId = choices.getBundle(CHOICE_CONTACT).getInt(ContactData.CONTACT_ID);

		// Otherwhise we need to ask which one to use.
		} else {
			// Prepare variables to ask to the user which contact to open.
			LinkedList<String> labelsList = new LinkedList<String>();
			LinkedList<Bundle> valuesList = new LinkedList<Bundle>();
			for (Bundle contactToCheck : contacts) {
				labelsList.add(contactToCheck.getString(ContactData.NAME));
				valuesList.add(contactToCheck);
			}

			// Setting the result code in order to enable the choice mode of Voice Control.
			setResultCode(VoiceControlPluginApi.ACTION_RESULT_ASK_CHOICE);

			// Setting choice data.
			resultExtras.putString(VoiceControlPluginApi.EXTRA_CHOICE_ID, CHOICE_CONTACT);
			resultExtras.putString(VoiceControlPluginApi.EXTRA_MESSAGE,
					context.getString(R.string.action_contact_choose_contact_name));
			String[] labels = labelsList.toArray(new String[labelsList.size()]);
			Bundle[] values = valuesList.toArray(new Bundle[valuesList.size()]);

			resultExtras.putStringArray(VoiceControlPluginApi.EXTRA_CHOICE_LABELS, labels);
			resultExtras.putParcelableArray(VoiceControlPluginApi.EXTRA_CHOICE_VALUES, values);

			// Quitting and waiting for the next invocation.
			return;
		}
		
		// Start the intent to open a contact detail from its id.
		Intent viewContactIntent = new Intent(Intent.ACTION_VIEW);
		Uri uri = Uri.withAppendedPath(ContactsContract.Contacts.CONTENT_URI, String.valueOf(contactId));
		viewContactIntent.setData(uri);
		viewContactIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		context.startActivity(viewContactIntent);

		// Setting the result result code for this BroadcastReceiver execution.
		setResultCode(VoiceControlPluginApi.ACTION_RESULT_OK);
	}

	public void handleCustomDataTypeAction(Context context, Intent intent) {
		Bundle resultExtras = getResultExtras(true);

		Bundle parameters = intent.getBundleExtra(VoiceControlPluginApi.EXTRA_PARAMETERS);

		// If the custom data value is missing, ask for it.
		if (!parameters.containsKey(PARAM_CUSTOM)) {
			setResultCode(VoiceControlPluginApi.ACTION_RESULT_ASK_PARAM);

			resultExtras.putString(VoiceControlPluginApi.EXTRA_PARAMETER_ID, PARAM_CUSTOM);

			return;
		}

		// We know that, for this plug-in, IDs and text are unique. Otherwise, we would handle it with a choice. 
		String valueId = parameters.getStringArrayList(PARAM_CUSTOM).get(0);

		// Do something plugin-oriented with the value id :)
		// ...
		// ...
		// At the moment, we only tell back which values has been used.
		resultExtras.putString(VoiceControlPluginApi.EXTRA_MESSAGE, 
				context.getString(R.string.action_custom_type_message, valueId));
		
		setResultCode(VoiceControlPluginApi.ACTION_RESULT_OK);
	}
}
